
local compost = AceLibrary("Compost-2.0")
local gratuity = AceLibrary("Gratuity-2.0")
local debuffTextureCache = {}
local buffTextureCache = {}

local vmajor, vminor = "Aura 1", tonumber(string.sub("$Revision: 9185 $", 12, -3))
local stubvarname = "TekLibStub"
local libvarname = "SpecialEventsEmbed"


-- Check to see if an update is needed
-- if not then just return out now before we do anything
local libobj = getglobal(libvarname)
if libobj and not libobj:NeedsUpgraded(vmajor, vminor) then return end


local stubobj = getglobal(stubvarname)
if not stubobj then
	stubobj = {}
	setglobal(stubvarname, stubobj)


	-- Instance replacement method, replace contents of old with that of new
	function stubobj:ReplaceInstance(old, new)
		 for k,v in pairs(old) do old[k]=nil end
		 for k,v in pairs(new) do old[k]=v end
	end


	-- Get a new copy of the stub
	function stubobj:NewStub(name)
		local newStub = {}
		self:ReplaceInstance(newStub, self)
		newStub.libName = name
		newStub.lastVersion = ''
		newStub.versions = {}
		return newStub
	end


	-- Get instance version
	function stubobj:NeedsUpgraded(vmajor, vminor)
		local versionData = self.versions[vmajor]
		if not versionData or versionData.minor < vminor then return true end
	end


	-- Get instance version
	function stubobj:GetInstance(version)
		if not version then version = self.lastVersion end
		local versionData = self.versions[version]
		if not versionData then print(string.format("<%s> Cannot find library version: %s", self.libName, version or "")) return end
		return versionData.instance
	end


	-- Register new instance
	function stubobj:Register(newInstance)
		 local version,minor = newInstance:GetLibraryVersion()
		 self.lastVersion = version
		 local versionData = self.versions[version]
		 if not versionData then
				-- This one is new!
				versionData = {
					instance = newInstance,
					minor = minor,
					old = {},
				}
				self.versions[version] = versionData
				newInstance:LibActivate(self)
				return newInstance
		 end
		 -- This is an update
		 local oldInstance = versionData.instance
		 local oldList = versionData.old
		 versionData.instance = newInstance
		 versionData.minor = minor
		 local skipCopy = newInstance:LibActivate(self, oldInstance, oldList)
		 table.insert(oldList, oldInstance)
		 if not skipCopy then
				for i, old in ipairs(oldList) do self:ReplaceInstance(old, newInstance) end
		 end
		 return newInstance
	end
end


if not libobj then
	libobj = stubobj:NewStub(libvarname)
	setglobal(libvarname, libobj)
end

local lib = {}


-- Return the library's current version
function lib:GetLibraryVersion()
	return vmajor, vminor
end


function lib:RegisterEvent(caller, eventname, method)
	if not eventname or not caller then return end

	if not self.registrar[eventname] then self.registrar[eventname] = {} end
	self.registrar[eventname][caller] = method or eventname
	return true
end


function lib:UnregisterEvent(caller, eventname)
	if not eventname or not caller or not self.registrar[eventname]
		or not self.registrar[eventname][caller] then return end

	self.registrar[eventname][caller] = nil
	return true
end


function lib:UnregisterAllEvents(caller)
	if not caller then return end

	for _,v in pairs(self.registrar) do v[caller] = nil end
	return true
end


function lib:TriggerEvent(eventname, a1,a2,a3,a4)
	if not eventname or not self.registrar[eventname] then return end

	for i,v in pairs(self.registrar[eventname]) do
		if type(v) == "function" then v(i,a1,a2,a3,a4)
		elseif i[v] then i[v](i,a1,a2,a3,a4) end
	end
end


-- Activate a new instance of this library
function lib:LibActivate(stub, oldLib, oldList)
	if oldLib then
		self.vars, self.registrar = oldLib.vars, oldLib.registrar
		self.frame = oldLib.frame
	else
		self.registrar, self.vars = {}, {}
		self.vars.buffs, self.vars.debuffs = {}, {}
		self.frame = CreateFrame("Frame")
		self.frame.name = "SEEmbed Aura Frame"
		self.frame:RegisterEvent("PLAYER_ENTERING_WORLD")
	end
	self.frame:SetScript("OnEvent", self.OnEvent)
end


function lib:OnEvent()
	local self = lib
	if self[event] then self[event](self) end
end


function lib:PLAYER_ENTERING_WORLD()
	self:ScanAllAuras()

	self.frame:RegisterEvent("PLAYER_LEAVING_WORLD")
	self.frame:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self.frame:RegisterEvent("RAID_ROSTER_UPDATE")
	self.frame:RegisterEvent("UNIT_AURA")
	self.frame:RegisterEvent("UNIT_AURASTATE")
	self.frame:RegisterEvent("PLAYER_AURAS_CHANGED")
	self.frame:RegisterEvent("PLAYER_TARGET_CHANGED")
end


function lib:PLAYER_LEAVING_WORLD()
	self.frame:UnregisterEvent("UNIT_AURA")
	self.frame:UnregisterEvent("UNIT_AURASTATE")
end


--------------------------------
--      Tracking methods      --
--------------------------------

function lib:UNIT_AURA()
	self:AuraScan(arg1)
end


function lib:UNIT_AURASTATE()
	self:AuraScan(arg1)
end


function lib:PLAYER_AURAS_CHANGED()
	self:AuraScan("player")
	if GetNumRaidMembers() > 0 then
		local u
		for i=1,GetNumRaidMembers() do
			if UnitIsUnit("raid"..i, "player") then u = "raid"..i end
		end
		self:AuraScan(u)
	end
end


function lib:PLAYER_TARGET_CHANGED()
	self:AuraScan("target")
	self:TriggerEvent("SPECIAL_AURA_TARGETCHANGED")
end


function lib:PARTY_MEMBERS_CHANGED()
	if (UnitExists("pet")) then self:AuraScan("pet") end

	for i=1,4 do
		if (UnitExists("party"..i)) then self:AuraScan("party"..i) end
		if (UnitExists("partypet"..i)) then self:AuraScan("partypet"..i) end
	end
	self:TriggerEvent("SPECIAL_AURA_PARTY_MEMBERS_CHANGED")
end


function lib:RAID_ROSTER_UPDATE()
	for i=1,40 do
		if (UnitExists("raid"..i)) then self:AuraScan("raid"..i) end
		if (UnitExists("raidpet"..i)) then self:AuraScan("raidpet"..i) end
	end
	self:TriggerEvent("SPECIAL_AURA_RAID_ROSTER_UPDATE")
end


function lib:ScanAllAuras()
	if (UnitExists("player")) then self:AuraScan("player") end
	if (UnitExists("pet")) then self:AuraScan("pet") end

	for i=1,4 do
		if (UnitExists("party"..i)) then self:AuraScan("party"..i) end
		if (UnitExists("partypet"..i)) then self:AuraScan("partypet"..i) end
	end

	for i=1,40 do
		if (UnitExists("raid"..i)) then self:AuraScan("raid"..i) end
		if (UnitExists("raidpet"..i)) then self:AuraScan("raidpet"..i) end
	end

	if (UnitExists("target")) then self:AuraScan("target") end
	if (UnitExists("mouseover")) then self:AuraScan("mouseover") end
end


function lib:AuraScan(targ)
	local maxbuffs, maxdebuffs, t = 16, 16, targ or arg1
	local oldd, oldb = self.vars.debuffs[t], self.vars.buffs[t]
	local newd, newb = compost:Acquire(), compost:Acquire()

	if t == "player" then
		for i=0,(maxbuffs-1) do
			local bidx = GetPlayerBuff(i, "HELPFUL")
			if bidx and bidx ~= -1 then
				gratuity:SetPlayerBuff(bidx)
				local txt = gratuity:GetLine(1)
				if txt then newb[txt] = i end
			end
		end

		for i=0,(maxdebuffs-1) do
			local didx = GetPlayerBuff(i, "HARMFUL")
			if didx and didx ~= -1 then
				gratuity:SetPlayerBuff(didx)
				local txt, txtr = gratuity:GetLine(1)

				local apps = GetPlayerBuffApplications(didx)
				local texture = GetPlayerBuffTexture(didx)
				local dbtype = GetPlayerBuffDispelType(didx)

				if txt then newd[txt] = compost:Acquire(i, apps or 1, dbtype, texture) end
			end
		end
	else
		for i=1,maxbuffs do
			local txt
			local texture, apps = UnitBuff (t, i)
			if texture then 
				if not buffTextureCache[texture] then
					gratuity:SetUnitBuff(t,i)
					txt = gratuity:GetLine(1)
					buffTextureCache[texture] = txt
				else
					txt = buffTextureCache[texture]
				end
				if txt then newb[txt] = i end
			end
		end
	
		for i=1,maxdebuffs do
			local txt
			local texture, apps, dbtype = UnitDebuff (t, i)
			if texture then
				if not debuffTextureCache[texture] then
					gratuity:SetUnitDebuff(t,i)
					txt = gratuity:GetLine(1)
					debuffTextureCache[texture] = txt
				else
					txt = debuffTextureCache[texture]
				end
				if txt then
					compost:Reclaim(newd[txt]) -- Sometimes we get the same debuff twice, SEAura doesn't distinguish multiples of the same debuff unless it stacks
					newd[txt] = compost:Acquire(i, apps or 1, dbtype, texture)
				end
			end
		end
	end

	self.vars.buffs[t] = newb
	self.vars.debuffs[t] = newd

	if oldb then
		for b,i in oldb do
			if not newb[b] then
				self:TriggerEvent("SPECIAL_UNIT_BUFF_LOST", t, b)
				if (t == "player") then self:TriggerEvent("SPECIAL_PLAYER_BUFF_LOST", b) end
			end
		end
	end

	if oldd then
		for d,i in pairs(oldd) do
			if type(i) ~= "table" then
				local t = ""
				for d,i in pairs(oldd) do t = t.." "..d..":"..i end
				error(string.format("Debuff error! Unit: %s, Table dump:%s", targ, t))
			elseif not newd[d] then
				self:TriggerEvent("SPECIAL_UNIT_DEBUFF_LOST", t, d, i[2], i[3], i[4])
				if (t == "player") then self:TriggerEvent("SPECIAL_PLAYER_DEBUFF_LOST", d, i[2], i[3], i[4]) end
			end
		end
	end

	for b,i in pairs(newb) do
		if (not oldb or not oldb[b]) then
			self:TriggerEvent("SPECIAL_UNIT_BUFF_GAINED", t, b)
			if (t == "player") then self:TriggerEvent("SPECIAL_PLAYER_BUFF_GAINED", b, i) end
		end
	end

	for d,i in pairs(newd) do
		assert(type(i) == "table", string.format("Debuff: %s - Value not a table: %s", d, type(i) == "table" and "table" or i or "nil"))

		local o2 = oldd and type(oldd[d]) == "table" and oldd[d][2]
		if not o2 or o2 ~= i[2] then
			self:TriggerEvent("SPECIAL_UNIT_DEBUFF_GAINED", t, d, i[2], i[3], i[4])
			if (t == "player") then self:TriggerEvent("SPECIAL_PLAYER_DEBUFF_GAINED", d, i[2], i[3], i[4]) end
		end
	end

	compost:ReclaimMulti(oldb, oldd, 1)
--	self.vars.buffs[t] = newb
--	self.vars.debuffs[t] = newd
end


-----------------------------
--      Query Methods      --
-----------------------------

function lib:UnitHasBuff(targ, buff)
	if self.vars.buffs[targ] then return self.vars.buffs[targ][buff] end
end


function lib:UnitHasDebuff(targ, debuff)
	if (self.vars.debuffs[targ] and self.vars.debuffs[targ][debuff]) then
		return self.vars.debuffs[targ][debuff][1], self.vars.debuffs[targ][debuff][2]
	end
end


function lib:UnitHasDebuffType(targ, debufftype)
	if self.vars.debuffs[targ] then
		for i,v in (self.vars.debuffs[targ]) do
			if v[3] == debufftype then return v end
		end
	end
end


--------------------------------
--      Load this bitch!      --
--------------------------------
libobj:Register(lib)
