
assert(CompostLib, "CompostLib not found")
assert(Gratuity, "Gratuity not found")
assert(SpecialEventsEmbed, "SpecialEventsEmbed not found")

local seaura = SpecialEventsEmbed:GetInstance("Aura 1")
assert(seaura, "SEEmbedAura not found")

local compost = CompostLib:GetInstance("compost-1")
local gratuity = Gratuity:GetInstance("1")

local vmajor, vminor = "Mount 1", tonumber(string.sub("$Revision: 7132 $", 12, -3))


-- Check to see if an update is needed
-- if not then just return out now before we do anything
if not SpecialEventsEmbed:NeedsUpgraded(vmajor, vminor) then return end

local loc = GetLocale() == "deDE" and {
	tooltipfind = "Erh\195\182\ht Tempo um (.+)%%",
} or {
	tooltipfind = "Increases speed by (.+)%%",
}


local lib = {}


-- Return the library's current version
function lib:GetLibraryVersion()
	return vmajor, vminor
end


function lib:RegisterEvent(caller, eventname, method)
	if not eventname or not caller then return end

	if not self.registrar[eventname] then self.registrar[eventname] = {} end
	self.registrar[eventname][caller] = method or eventname
	return true
end


function lib:UnregisterEvent(caller, eventname)
	if not eventname or not caller or not self.registrar[eventname]
		or not self.registrar[eventname][caller] then return end

	self.registrar[eventname][caller] = nil
	return true
end


function lib:UnregisterAllEvents(caller)
	if not caller then return end

	for _,v in pairs(self.registrar) do v[caller] = nil end
	return true
end


function lib:TriggerEvent(eventname, a1,a2,a3,a4)
	if not eventname or not self.registrar[eventname] then return end

	for i,v in pairs(self.registrar[eventname]) do
		if type(v) == "function" then v(i,a1,a2,a3,a4)
		elseif i[v] then i[v](i,a1,a2,a3,a4) end
	end
end


-- Activate a new instance of this library
function lib:LibActivate(stub, oldLib, oldList)
	if oldLib then
		self.vars, self.registrar = oldLib.vars, oldLib.registrar
	else
		self.registrar, self.vars = {}, {}
		self:ScanAllAuras()
		seaura:RegisterEvent(self, "SPECIAL_PLAYER_BUFF_GAINED")
		seaura:RegisterEvent(self, "SPECIAL_PLAYER_BUFF_LOST")
	end
end


function lib:ScanAllAuras()
	if seaura.vars.buffs and seaura.vars.buffs.player then
		for b,i in pairs(seaura.vars.buffs.player) do
			self:SPECIAL_PLAYER_BUFF_GAINED(b, i)
		end
	end
end


function lib:SPECIAL_PLAYER_BUFF_GAINED(buff, i)
	if UnitOnTaxi("player") then return end

	gratuity:SetPlayerBuff(GetPlayerBuff(i, "HELPFUL"))
	local txt, speed = gratuity:GetLine(2)
	if txt then _, _, speed = string.find(txt, loc.tooltipfind) end

	if (speed and not self.vars.mounted) then
		self.vars.mounted, self.vars.mountspeed = buff, speed
		self:TriggerEvent("SPECIAL_PLAYER_MOUNTED", buff, speed, i)
	end
end


function lib:SPECIAL_PLAYER_BUFF_LOST(buff)
	if buff == self.vars.mounted then
		local s = self.vars.mountspeed
		self.vars.mounted, self.vars.mountspeed = nil, nil
		self:TriggerEvent("SPECIAL_PLAYER_DISMOUNTED", buff, s)
	end
end


-----------------------------
--      Query Methods      --
-----------------------------

function lib:PlayerMounted()
	return self.vars.mounted, self.vars.mountspeed
end


--------------------------------
--      Load this bitch!      --
--------------------------------
SpecialEventsEmbed:Register(lib)
